/* A module that implements a somewhat more sophisticated string class */

#include <stdlib.h>
#include <string.h>
#include "edelstring.h"

eStr *eStr_new(int len)
/* returns a freshly allocated eStr for up to len characters.  It
is initially empty (i.e. has length 0).   Returns NULL on error. */
{
	eStr *eString = malloc(sizeof(eStr));

	if (eString) {
		eString->length = len;
		if (!(eString->contents = malloc((len+1)*sizeof(char)))) {
			free(eString);
			return NULL;
		}
		memset(eString->contents, 0, len+1);
		eString->cCompatible = 0;
	}
	return eString;
}

eStr *eStr_fromString(char *cString)
/* returns a freshly allocated eStr, constructed from a conventional
zero-terminated C string.  Its capacity is equal to the C string's
length. */
{
	eStr *eString=eStr_new(strlen(cString));

	if (eString) {
		strcpy(eString->contents, cString);
		eString->cCompatible = 1;
	}
	return eString;
}

void eStr_free(eStr *eString)
/* releases the memory allocated vor the eStr. */
{
	free(eString->contents);
	free(eString);
}
	
eStr *eStr_add(eStr *op1, eStr *op2)
/* returns an new eStr that is the concatenation of op1 and op2 or
NULL if there is an error. */
{
	eStr *dest=eStr_new(op1->length+op2->length);

	if (dest) {
		memcpy(dest->contents, op1->contents, op1->length);
		memcpy(dest->contents+op1->length, op2->contents, op2->length);
	}
	return dest;
}

int eStr_findChar(eStr *eString, char c, int count)
/* returns the index of the counteth occurrence of c in eString or
-1 if it doesn't exist. */
{
	int i;

	for (i=0; i<eString->length; i++) {
		if (eString->contents[i]==c) {
			if (!--count) {
				return i;
			}
		}
	}
	return -1;
}

char *eStr_asString(eStr *eString)
{
	return eString->contents;
}
